@@ -1,10 +1,9 @@
 package Net::DNS::Mailbox;
 
-#
-# $Id: Mailbox.pm 1339 2015-04-16 13:01:17Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1339 $)[1];
+use strict;
+use warnings;
+
+our $VERSION = (qw$Id: Mailbox.pm 1990 2024-09-18 13:16:07Z willem $)[2];
 
 
 =head1 NAME
@@ -15,7 +14,7 @@ Net::DNS::Mailbox - DNS mailbox representation
 
     use Net::DNS::Mailbox;
 
-    $mailbox = new Net::DNS::Mailbox('user@example.com');
+    $mailbox = Net::DNS::Mailbox->new('user@example.com');
     $address = $mailbox->address;
 
 =head1 DESCRIPTION
@@ -23,10 +22,15 @@ Net::DNS::Mailbox - DNS mailbox representation
 The Net::DNS::Mailbox module implements a subclass of DNS domain name
 objects representing the DNS coded form of RFC822 mailbox address.
 
+The Net::DNS::Mailbox1035 and Net::DNS::Mailbox2535 packages
+implement mailbox representation subtypes which provide the name
+compression and canonicalisation specified by RFC1035 and RFC2535.
+These are necessary to meet the backward compatibility requirements
+introduced by RFC3597.
+
 =cut
 
 
-use strict;
 use integer;
 use Carp;
 
@@ -37,11 +41,13 @@ use base qw(Net::DNS::DomainName);
 
 =head2 new
 
-    $mailbox = new Net::DNS::Mailbox('John.Doe@example.com');
-    $mailbox = new Net::DNS::Mailbox('John Doe <j.doe@example.com>');
+    $mailbox = Net::DNS::Mailbox->new('John Doe <john.doe@example.com>');
+    $mailbox = Net::DNS::Mailbox->new('john.doe@example.com');
+    $mailbox = Net::DNS::Mailbox->new('john\.doe.example.com');
 
-Creates a mailbox object which represents the DNS domain encoded form
-of the mail address specified by the character string argument.
+Creates a mailbox object representing the RFC822 mail address specified by
+the character string argument. An encoded domain name is also accepted for
+backward compatibility with Net::DNS 0.68 and earlier.
 
 The argument string consists of printable characters from the 7-bit
 ASCII repertoire.
@@ -55,18 +61,13 @@ sub new {
 
 	s/^.*<//g;						# strip excess on left
 	s/>.*$//g;						# strip excess on right
+	s/^\@.+://;						# strip deprecated source route
+	s/\\\./\\046/g;						# disguise escaped dots
 
-	s/\\\@/\\064/g;						# disguise escaped @
-	s/^(".*)\@(.*")/$1\\064$2/g;				# disguise quoted @
+	my ( $localpart, @domain ) = split /[@.]([^@;:"]*$)/;	# split on rightmost @
+	s/\./\\046/g for $localpart ||= '';			# escape dots in local part
 
-	my ( $mbox, @host ) = split /\@/;			# split on @ if present
-	for ( $mbox ||= '' ) {
-		s/^"(.*)"/$1/;					# strip quotes
-		s/\\\./\\046/g;					# disguise escaped dot
-		s/\./\\046/g if @host;				# escape dots in local part
-	}
-
-	bless __PACKAGE__->SUPER::new( join '.', $mbox, @host ), $class;
+	return bless __PACKAGE__->SUPER::new( join '.', $localpart, @domain ), $class;
 }
 
 
@@ -81,37 +82,31 @@ in RFC1035 section 8.
 =cut
 
 sub address {
+	return unless defined wantarray;
 	my @label = shift->label;
 	local $_ = shift(@label) || return '<>';
+	s/\\\\//g;						# delete escaped \
+	s/^\\034(.*)\\034$/"$1"/;				# unescape enclosing quotes
+	s/\\\d\d\d//g;						# delete non-printable
 	s/\\\./\./g;						# unescape dots
-	s/\\032/ /g;						# unescape space
-	s/^(.+)$/"$1"/ if /[ ",@\[\\\]]/;			# quote local part
-	return join '@', $_, join( '.', @label ) || ();
+	s/\\//g;						# delete escapes
+	return $_ unless scalar(@label);
+	return join '@', $_, join '.', @label;
 }
 
 
 ########################################
 
-=head1 DOMAIN NAME COMPRESSION AND CANONICALISATION
+package Net::DNS::Mailbox1035;		## no critic ProhibitMultiplePackages
+our @ISA = qw(Net::DNS::Mailbox);
 
-The Net::DNS::Mailbox1035 and Net::DNS::Mailbox2535 subclass
-packages implement RFC1035 domain name compression and RFC2535
-canonicalisation.
-
-=cut
-
-package Net::DNS::Mailbox1035;
-
-use base qw(Net::DNS::Mailbox);
-
-sub encode { &Net::DNS::DomainName1035::encode; }
+sub encode { return &Net::DNS::DomainName1035::encode; }
 
 
-package Net::DNS::Mailbox2535;
+package Net::DNS::Mailbox2535;		## no critic ProhibitMultiplePackages
+our @ISA = qw(Net::DNS::Mailbox);
 
-use base qw(Net::DNS::Mailbox);
-
-sub encode { &Net::DNS::DomainName2535::encode; }
+sub encode { return &Net::DNS::DomainName2535::encode; }
 
 
 1;
@@ -131,7 +126,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -148,7 +143,9 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::DomainName>, RFC822, RFC1035, RFC5322
+L<perl> L<Net::DNS> L<Net::DNS::DomainName>
+L<RFC1035|https://iana.org/go/rfc1035>
+L<RFC5322|https://iana.org/go/rfc5322>
 
 =cut
 
