@@ -3,7 +3,7 @@ package Net::DNS::Packet;
 use strict;
 use warnings;
 
-our $VERSION = (qw$Id: Packet.pm 1895 2023-01-16 13:38:08Z willem $)[2];
+our $VERSION = (qw$Id: Packet.pm 2000 2025-01-02 16:34:13Z willem $)[2];
 
 
 =head1 NAME
@@ -12,11 +12,11 @@ Net::DNS::Packet - DNS protocol packet
 
 =head1 SYNOPSIS
 
-    use Net::DNS::Packet;
+ use Net::DNS::Packet;
 
-    $query = Net::DNS::Packet->new( 'example.com', 'MX', 'IN' );
+ $query = Net::DNS::Packet->new( 'example.com', 'MX', 'IN' );
 
-    $reply = $resolver->send( $query );
+ $reply = $resolver->send( $query );
 
 
 =head1 DESCRIPTION
@@ -43,10 +43,10 @@ BEGIN {
 
 =head2 new
 
-    $packet = Net::DNS::Packet->new( 'example.com' );
-    $packet = Net::DNS::Packet->new( 'example.com', 'MX', 'IN' );
+ $packet = Net::DNS::Packet->new( 'example.com' );
+ $packet = Net::DNS::Packet->new( 'example.com', 'MX', 'IN' );
 
-    $packet = Net::DNS::Packet->new();
+ $packet = Net::DNS::Packet->new();
 
 If passed a domain, type, and class, new() creates a Net::DNS::Packet
 object which is suitable for making a DNS query for the specified
@@ -75,16 +75,14 @@ sub new {
 }
 
 
-#=head2 decode
+=head2 decode
 
-=pod
+ $packet = Net::DNS::Packet->decode( \$data );
+ $packet = Net::DNS::Packet->decode( \$data, 1 );	# debug
+ $packet = Net::DNS::Packet->new( \$data ... );
 
-    $packet = Net::DNS::Packet->decode( \$data );
-    $packet = Net::DNS::Packet->decode( \$data, 1 );	# debug
-    $packet = Net::DNS::Packet->new( \$data ... );
-
-If passed a reference to a scalar containing DNS packet data, a new
-packet object is created by decoding the data.
+A new packet object is created by decoding the DNS packet data
+contained in the scalar referenced by the first argument.
 The optional second boolean argument enables debugging output.
 
 Returns undef if unable to create a packet object.
@@ -93,7 +91,7 @@ Decoding errors, including data corruption and truncation, are
 collected in the $@ ($EVAL_ERROR) variable.
 
 
-    ( $packet, $length ) = Net::DNS::Packet->decode( \$data );
+ ( $packet, $length ) = Net::DNS::Packet->decode( \$data );
 
 If called in array context, returns a packet object and the number
 of octets successfully decoded.
@@ -107,7 +105,7 @@ or corrupted during transmission.
 use constant HEADER_LENGTH => length pack 'n6', (0) x 6;
 
 sub decode {
-	my $class = shift;					# uncoverable pod
+	my $class = shift;
 	my $data  = shift;
 	my $debug = shift || 0;
 
@@ -180,10 +178,10 @@ sub decode {
 }
 
 
-=head2 data
+=head2 encode
 
-    $data = $packet->data;
-    $data = $packet->data( $size );
+ $data = $packet->encode;
+ $data = $packet->encode( $size );
 
 Returns the packet data in binary format, suitable for sending as a
 query or update request to a nameserver.
@@ -193,11 +191,11 @@ Truncation may be specified using a non-zero optional size argument.
 =cut
 
 sub data {
-	return &encode;
+	return &encode;						# uncoverable pod
 }
 
 sub encode {
-	my ( $self, $size ) = @_;				# uncoverable pod
+	my ( $self, $size ) = @_;
 
 	my $edns = $self->edns;					# EDNS support
 	my @addl = grep { !$_->isa('Net::DNS::RR::OPT') } @{$self->{additional}};
@@ -207,7 +205,7 @@ sub encode {
 
 	my @part = qw(question answer authority additional);
 	my @size = map { scalar @{$self->{$_}} } @part;
-	my $data = pack 'n6', $self->header->id, $self->{status}, @size;
+	my $data = pack 'n6', $self->_quid, $self->{status}, @size;
 	$self->{count} = [];
 
 	my $hash = {};						# packet body
@@ -221,7 +219,7 @@ sub encode {
 
 =head2 header
 
-    $header = $packet->header;
+ $header = $packet->header;
 
 Constructor method which returns a Net::DNS::Header object which
 represents the header section of the packet.
@@ -236,9 +234,8 @@ sub header {
 
 =head2 edns
 
-    $edns    = $packet->edns;
-    $version = $edns->version;
-    $UDPsize = $edns->size;
+ $version = $packet->edns->version;
+ $UDPsize = $packet->edns->size;
 
 Auxiliary function which provides access to the EDNS protocol
 extension OPT RR.
@@ -256,7 +253,7 @@ sub edns {
 
 =head2 reply
 
-    $reply = $query->reply( $UDPmax );
+ $reply = $query->reply( $UDPmax );
 
 Constructor method which returns a new reply packet.
 
@@ -267,8 +264,7 @@ response to an EDNS query.
 =cut
 
 sub reply {
-	my $query  = shift;
-	my $UDPmax = shift;
+	my ( $query, @UDPmax ) = @_;
 	my $qheadr = $query->header;
 	croak 'erroneous qr flag in query packet' if $qheadr->qr;
 
@@ -289,14 +285,14 @@ sub reply {
 
 	my $edns = $reply->edns();
 	CORE::push( @{$reply->{additional}}, $edns );
-	$edns->size($UDPmax);
+	$edns->udpsize(@UDPmax);
 	return $reply;
 }
 
 
 =head2 question, zone
 
-    @question = $packet->question;
+ @question = $packet->question;
 
 Returns a list of Net::DNS::Question objects representing the
 question section of the packet.
@@ -316,7 +312,7 @@ sub zone { return &question }
 
 =head2 answer, pre, prerequisite
 
-    @answer = $packet->answer;
+ @answer = $packet->answer;
 
 Returns a list of Net::DNS::RR objects representing the answer
 section of the packet.
@@ -338,7 +334,7 @@ sub prerequisite { return &answer }
 
 =head2 authority, update
 
-    @authority = $packet->authority;
+ @authority = $packet->authority;
 
 Returns a list of Net::DNS::RR objects representing the authority
 section of the packet.
@@ -358,7 +354,7 @@ sub update { return &authority }
 
 =head2 additional
 
-    @additional = $packet->additional;
+ @additional = $packet->additional;
 
 Returns a list of Net::DNS::RR objects representing the additional
 section of the packet.
@@ -373,7 +369,7 @@ sub additional {
 
 =head2 print
 
-    $packet->print;
+ $packet->print;
 
 Prints the entire packet to the currently selected output filehandle
 using the master file format mandated by RFC1035.
@@ -388,7 +384,7 @@ sub print {
 
 =head2 string
 
-    print $packet->string;
+ print $packet->string;
 
 Returns a string representation of the packet.
 
@@ -399,23 +395,24 @@ sub string {
 
 	my $header = $self->header;
 	my $opcode = $header->opcode;
+	my $packet = $header->qr ? 'Response' : 'Query';
 	my $server = $self->{replyfrom};
 	my $length = $self->{replysize};
-	my $origin = $server ? ";; Response received from $server ($length octets)\n" : "";
+	my $origin = $server ? ";; $packet received from [$server] $length octets\n" : "";
 	my @record = ( "$origin;; HEADER SECTION", $header->string );
 
-	my $edns = $self->edns;
-	CORE::push( @record, $edns->string ) if $edns->_specified;
-
 	if ( $opcode eq 'DSO' ) {
 		CORE::push( @record, ";; DSO SECTION" );
 		foreach ( @{$self->{dso}} ) {
 			my ( $t, $v ) = @$_;
-			CORE::push( @record, pack 'a* A18 a*', ";;\t", dsotypebyval($t), unpack( 'H*', $v ) );
+			CORE::push( @record, sprintf( ";;\t%s\t%s", dsotypebyval($t), unpack( 'H*', $v ) ) );
 		}
 		return join "\n", @record, "\n";
 	}
 
+	my $edns = $self->edns;
+	CORE::push( @record, $edns->string ) if $edns->_specified;
+
 	my @section  = $opcode eq 'UPDATE' ? qw(ZONE PREREQUISITE UPDATE) : qw(QUESTION ANSWER AUTHORITY);
 	my @question = $self->question;
 	my $qdcount  = scalar @question;
@@ -435,7 +432,7 @@ sub string {
 	my @additional = $self->additional;
 	my $arcount    = scalar @additional;
 	my $ars	       = $arcount != 1 ? 's' : '';
-	my $EDNSmarker = join ' ', qq[;; {\t"EDNS-VERSION":], $edns->version, qq[}\n];
+	my $EDNSmarker = join ' ', qq[;; {\t"EDNS-VERSION":], $edns->version, qq[}];
 	CORE::push( @record, "\n;; ADDITIONAL SECTION ($arcount record$ars)" );
 	CORE::push( @record, map { ( $_ eq $edns ) ? $EDNSmarker : $_->string } @additional );
 
@@ -463,7 +460,7 @@ sub answerfrom { return &from; }				# uncoverable pod
 
 =head2 size
 
-    print "packet size: ", $packet->size, " octets\n";
+ print "packet size: ", $packet->size, " octets\n";
 
 Returns the size of the packet in octets as it was received from a
 nameserver.  This method will return undef for user-created packets
@@ -480,12 +477,12 @@ sub answersize { return &size; }				# uncoverable pod
 
 =head2 push
 
-    $ancount = $packet->push( prereq => $rr );
-    $nscount = $packet->push( update => $rr );
-    $arcount = $packet->push( additional => $rr );
+ $ancount = $packet->push( prereq => $rr );
+ $nscount = $packet->push( update => $rr );
+ $arcount = $packet->push( additional => $rr );
 
-    $nscount = $packet->push( update => $rr1, $rr2, $rr3 );
-    $nscount = $packet->push( update => @rr );
+ $nscount = $packet->push( update => $rr1, $rr2, $rr3 );
+ $nscount = $packet->push( update => @rr );
 
 Adds RRs to the specified section of the packet.
 
@@ -504,12 +501,12 @@ sub push {
 
 =head2 unique_push
 
-    $ancount = $packet->unique_push( prereq => $rr );
-    $nscount = $packet->unique_push( update => $rr );
-    $arcount = $packet->unique_push( additional => $rr );
+ $ancount = $packet->unique_push( prereq => $rr );
+ $nscount = $packet->unique_push( update => $rr );
+ $arcount = $packet->unique_push( additional => $rr );
 
-    $nscount = $packet->unique_push( update => $rr1, $rr2, $rr3 );
-    $nscount = $packet->unique_push( update => @rr );
+ $nscount = $packet->unique_push( update => $rr1, $rr2, $rr3 );
+ $nscount = $packet->unique_push( update => @rr );
 
 Adds RRs to the specified section of the packet provided that the
 RRs are not already present in the same section.
@@ -531,9 +528,9 @@ sub unique_push {
 
 =head2 pop
 
-    my $rr = $packet->pop( 'pre' );
-    my $rr = $packet->pop( 'update' );
-    my $rr = $packet->pop( 'additional' );
+ my $rr = $packet->pop( 'pre' );
+ my $rr = $packet->pop( 'update' );
+ my $rr = $packet->pop( 'additional' );
 
 Removes a single RR from the specified section of the packet.
 
@@ -564,16 +561,16 @@ sub _section {				## returns array reference for section
 
 =head2 sign_tsig
 
-    $query = Net::DNS::Packet->new( 'www.example.com', 'A' );
+ $query = Net::DNS::Packet->new( 'www.example.com', 'A' );
 
-    $query->sign_tsig(
+ $query->sign_tsig(
 		$keyfile,
 		fudge => 60
 		);
 
-    $reply = $res->send( $query );
+ $reply = $res->send( $query );
 
-    $reply->verify( $query ) || die $reply->verifyerr;
+ $reply->verify( $query ) || die $reply->verifyerr;
 
 Attaches a TSIG resource record object, which will be used to sign
 the packet (see RFC 2845).
@@ -587,7 +584,7 @@ must uniquely identify the key shared between the parties, and the
 algorithm name must identify the signing function to be used with the
 specified key.
 
-    $tsig = Net::DNS::RR->new(
+ $tsig = Net::DNS::RR->new(
 		name		=> 'tsig.example',
 		type		=> 'TSIG',
 		algorithm	=> 'custom-algorithm',
@@ -598,22 +595,22 @@ specified key.
 					}
 		);
 
-    $query->sign_tsig( $tsig );
+ $query->sign_tsig( $tsig );
 
 
 The response to an inbound request is signed by presenting the request
 in place of the key parameter.
 
-    $response = $request->reply;
-    $response->sign_tsig( $request, @options );
+ $response = $request->reply;
+ $response->sign_tsig( $request, @options );
 
 
 Multi-packet transactions are signed by chaining the sign_tsig()
 calls together as follows:
 
-    $opaque  =	$packet1->sign_tsig( 'Kexample.+165+13281.private' );
-    $opaque  =	$packet2->sign_tsig( $opaque );
-		$packet3->sign_tsig( $opaque );
+ $opaque = $packet1->sign_tsig( 'Kexample.+165+13281.private' );
+ $opaque = $packet2->sign_tsig( $opaque );
+ $opaque = $packet3->sign_tsig( $opaque );
 
 The opaque intermediate object references returned during multi-packet
 signing are not intended to be accessed by the end-user application.
@@ -638,15 +635,15 @@ sub sign_tsig {
 
 =head2 verify and verifyerr
 
-    $packet->verify()		|| die $packet->verifyerr;
-    $reply->verify( $query )	|| die $reply->verifyerr;
+ $packet->verify()	|| die $packet->verifyerr;
+ $reply->verify($query)	|| die $reply->verifyerr;
 
 Verify TSIG signature of packet or reply to the corresponding query.
 
 
-    $opaque  =	$packet1->verify( $query ) || die $packet1->verifyerr;
-    $opaque  =	$packet2->verify( $opaque );
-    $verifed =	$packet3->verify( $opaque ) || die $packet3->verifyerr;
+ $opaque  = $packet1->verify( $query ) || die $packet1->verifyerr;
+ $opaque  = $packet2->verify( $opaque );
+ $verifed = $packet3->verify( $opaque ) || die $packet3->verifyerr;
 
 The opaque intermediate object references returned during multi-packet
 verify() will be undefined (Boolean false) if verification fails.
@@ -675,17 +672,17 @@ The requisite cryptographic components are not integrated into
 Net::DNS but reside in the Net::DNS::SEC distribution available
 from CPAN.
 
-    $update = Net::DNS::Update->new('example.com');
-    $update->push( update => rr_add('foo.example.com A 10.1.2.3'));
-    $update->sign_sig0('Kexample.com+003+25317.private');
+ $update = Net::DNS::Update->new('example.com');
+ $update->push( update => rr_add('foo.example.com A 10.1.2.3'));
+ $update->sign_sig0('Kexample.com+003+25317.private');
 
 Execution will be terminated if Net::DNS::SEC is not available.
 
 
 =head2 verify SIG0
 
-    $packet->verify( $keyrr )		|| die $packet->verifyerr;
-    $packet->verify( [$keyrr, ...] )	|| die $packet->verifyerr;
+ $packet->verify( $keyrr )		|| die $packet->verifyerr;
+ $packet->verify( [$keyrr, ...] )	|| die $packet->verifyerr;
 
 Verify SIG0 packet signature against one or more specified KEY RRs.
 
@@ -715,7 +712,7 @@ sub sign_sig0 {
 
 =head2 sigrr
 
-    $sigrr = $packet->sigrr() || die 'unsigned packet';
+ $sigrr = $packet->sigrr() || die 'unsigned packet';
 
 The sigrr method returns the signature RR from a signed packet
 or undefined if the signature is absent.
@@ -727,8 +724,9 @@ sub sigrr {
 
 	my ($sig) = reverse $self->additional;
 	return unless $sig;
-	return $sig if $sig->type eq 'TSIG';
-	return $sig if $sig->type eq 'SIG';
+	for ( $sig->type ) {
+		return $sig if /TSIG|SIG/;
+	}
 	return;
 }
 
@@ -823,7 +821,7 @@ sub truncate {
 
 	my @part = qw(question answer authority additional);
 	my @size = map { scalar @{$self->{$_}} } @part;
-	return pack 'n6 a*', $self->header->id, $self->{status}, @size, substr( $data, HEADER_LENGTH );
+	return pack 'n6 a*', $self->_quid, $self->{status}, @size, substr( $data, HEADER_LENGTH );
 }
 
 
@@ -840,6 +838,20 @@ sub dump {				## print internal data structure
 }
 
 
+my ( $cache1, $cache2, $limit );
+
+sub _quid {				## generate (short-term) unique query ID
+	my $self = shift;
+	my $id	 = $self->{id};
+	$cache1->{$id}++ if $id;				# cache non-zero ID
+	return $id	 if defined $id;
+	( $cache2, $cache1, $limit ) = ( $cache1, {0 => 1}, 50 ) unless $limit--;
+	$id = int rand(0xffff);					# two layer ID cache
+	$id = int rand(0xffff) while $cache1->{$id}++ + exists( $cache2->{$id} );
+	return $self->{id} = $id;
+}
+
+
 1;
 __END__
 
@@ -878,11 +890,12 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl> L<Net::DNS> L<Net::DNS::Update> L<Net::DNS::Header>
+L<perl> L<Net::DNS> L<Net::DNS::Header>
 L<Net::DNS::Question> L<Net::DNS::RR> L<Net::DNS::RR::TSIG>
-L<RFC1035(4.1)|https://tools.ietf.org/html/rfc1035>
-L<RFC2136(2)|https://tools.ietf.org/html/rfc2136>
-L<RFC8945|https://tools.ietf.org/html/rfc8945>
+L<RFC1035(4.1)|https://iana.org/go/rfc1035#section-4.1>
+L<RFC2136|https://iana.org/go/rfc2136#section-2>
+L<RFC8490|https://iana.org/go/rfc8490>
+L<RFC8945|https://iana.org/go/rfc8945>
 
 =cut
 
