@@ -1,60 +1,55 @@
 package Net::DNS::RR::EUI48;
 
-#
-# $Id: EUI48.pm 1382 2015-08-26 20:39:33Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
-
-
 use strict;
+use warnings;
+our $VERSION = (qw$Id: EUI48.pm 1896 2023-01-30 12:59:25Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::EUI48 - DNS EUI48 resource record
 
 =cut
 
-
 use integer;
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
-	my ( $data, $offset ) = @_;
+	my ( $self, $data, $offset ) = @_;
 
 	$self->{address} = unpack "\@$offset a6", $$data;
+	return;
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless $self->{address};
-	pack 'a6', $self->{address};
+	return pack 'a6', $self->{address};
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{address};
 	return $self->address;
 }
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	$self->address(shift);
+	$self->address(@argument);
+	return;
 }
 
 
 sub address {
 	my ( $self, $address ) = @_;
-	$self->{address} = pack 'C6', map hex($_), split /[:-]/, $address if $address;
-	join '-', unpack 'H2H2H2H2H2H2', $self->{address} if defined wantarray;
+	$self->{address} = pack 'C6', map { hex($_) } split /[:-]/, $address if $address;
+	return defined(wantarray) ? join( '-', unpack 'H2H2H2H2H2H2', $self->{address} ) : undef;
 }
 
 
@@ -65,9 +60,9 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name IN EUI48 address');
+    $rr = Net::DNS::RR->new('name IN EUI48 address');
 
-    $rr = new Net::DNS::RR(
+    $rr = Net::DNS::RR->new(
 	name	=> 'example.com',
 	type	=> 'EUI48',
 	address => '00-00-5e-00-53-2a'
@@ -113,7 +108,7 @@ Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -130,6 +125,7 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC7043
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<RFC7043|https://tools.ietf.org/html/rfc7043>
 
 =cut
