@@ -2,7 +2,7 @@ package Net::DNS::RR::MB;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: MB.pm 1896 2023-01-30 12:59:25Z willem $)[2];
+our $VERSION = (qw$Id: MB.pm 1972 2024-04-21 08:13:19Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -29,16 +29,14 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my ( $self, @argument ) = @_;
 
-	my $madname = $self->{madname} || return '';
-	return $madname->encode(@argument);
+	return $self->{madname}->encode(@argument);
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my $madname = $self->{madname} || return '';
-	return $madname->string;
+	return $self->{madname}->string;
 }
 
 
@@ -120,6 +118,6 @@ DEALINGS IN THE SOFTWARE.
 =head1 SEE ALSO
 
 L<perl> L<Net::DNS> L<Net::DNS::RR>
-L<RFC1035(3.3.3)|https://tools.ietf.org/html/rfc1035>
+L<RFC1035(3.3.3)|https://iana.org/go/rfc1035#section-3.3.3>
 
 =cut
