@@ -2,7 +2,7 @@ package Net::DNS::RR::TXT;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: TXT.pm 1857 2021-12-07 13:38:02Z willem $)[2];
+our $VERSION = (qw$Id: TXT.pm 1972 2024-04-21 08:13:19Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -22,18 +22,16 @@ use Net::DNS::Text;
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
-	my $self = shift;
-	my ( $data, $offset ) = @_;
+	my ( $self, $data, $offset ) = @_;
 
-	my $limit = $offset + $self->{rdlength};
-	my $text;
-	my $txtdata = $self->{txtdata} = [];
-	while ( $offset < $limit ) {
-		( $text, $offset ) = Net::DNS::Text->decode( $data, $offset );
-		push @$txtdata, $text;
+	my $limit = $self->{rdlength};
+	my $rdata = substr $$data, $offset, $limit;
+	my $array = $self->{txtdata} = [];
+	my $index = 0;
+	while ( $index < $limit ) {
+		( my $text, $index ) = Net::DNS::Text->decode( \$rdata, $index );
+		push @$array, $text;
 	}
-
-	croak('corrupt TXT data') unless $offset == $limit;	# more or less FUBAR
 	return;
 }
 
@@ -55,17 +53,17 @@ sub _format_rdata {			## format rdata portion of RR string.
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
-	my $self = shift;
+	my ( $self, @argument ) = @_;
 
-	$self->{txtdata} = [map { Net::DNS::Text->new($_) } @_];
+	$self->{txtdata} = [map { Net::DNS::Text->new($_) } @argument];
 	return;
 }
 
 
 sub txtdata {
-	my $self = shift;
+	my ( $self, @value ) = @_;
 
-	$self->{txtdata} = [map { Net::DNS::Text->new($_) } @_] if scalar @_;
+	$self->{txtdata} = [map { Net::DNS::Text->new($_) } @value] if scalar @value;
 
 	my $txtdata = $self->{txtdata} || [];
 
@@ -75,7 +73,7 @@ sub txtdata {
 }
 
 
-sub char_str_list { return (&txtdata); }			# uncoverable pod
+sub char_str_list { return my @txt = &txtdata }			# uncoverable pod
 
 
 1;
@@ -158,6 +156,8 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC1035 Section 3.3.14, RFC3629
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<RFC1035(3.3.14)|https://iana.org/go/rfc1035#section-3.3.14>
+L<RFC3629|https://iana.org/go/rfc3629>
 
 =cut
