@@ -3,7 +3,7 @@ package Net::DNS::Resolver;
 use strict;
 use warnings;
 
-our $VERSION = (qw$Id: Resolver.pm 1813 2020-10-08 21:58:40Z willem $)[2];
+our $VERSION = (qw$Id: Resolver.pm 2000 2025-01-02 16:34:13Z willem $)[2];
 
 
 =head1 NAME
@@ -13,11 +13,7 @@ Net::DNS::Resolver - DNS resolver class
 =cut
 
 
-use constant CONFIG => defined eval "require Net::DNS::Resolver::$^O";	  ## no critic
-
-use constant OS_CONF => join '::', __PACKAGE__, CONFIG ? $^O : 'UNIX';
-
-use base OS_CONF;
+use base qw(Net::DNS::Resolver::Base);
 
 
 1;
@@ -27,22 +23,22 @@ __END__
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
+ use Net::DNS;
 
-    $resolver = Net::DNS::Resolver->new();
+ $resolver = Net::DNS::Resolver->new();
 
-    # Perform a lookup, using the searchlist if appropriate.
-    $reply = $resolver->search( 'example.com' );
+ # Perform a lookup, using the searchlist if appropriate.
+ $reply = $resolver->search( 'example.com' );
 
-    # Perform a lookup, without the searchlist
-    $reply = $resolver->query( 'example.com', 'MX' );
+ # Perform a lookup, without the searchlist
+ $reply = $resolver->query( 'example.com', 'MX' );
 
-    # Perform a lookup, without pre or post-processing
-    $reply = $resolver->send( 'example.com', 'MX', 'IN' );
+ # Perform a lookup, without pre or post-processing
+ $reply = $resolver->send( 'example.com', 'MX', 'IN' );
 
-    # Send a prebuilt query packet
-    $query = Net::DNS::Packet->new( ... );
-    $reply = $resolver->send( $query );
+ # Send a prebuilt query packet
+ $query = Net::DNS::Packet->new( ... );
+ $reply = $resolver->send( $query );
 
 =head1 DESCRIPTION
 
@@ -55,18 +51,18 @@ recursion is desired, etc.
 
 =head2 new
 
-    # Use the default configuration
-    $resolver = Net::DNS::Resolver->new();
+ # Use the default configuration
+ $resolver = Net::DNS::Resolver->new();
 
-    # Use my own configuration file
-    $resolver = Net::DNS::Resolver->new( config_file => '/my/dns.conf' );
+ # Use my own configuration file
+ $resolver = Net::DNS::Resolver->new( config_file => '/my/dns.conf' );
 
-    # Set options in the constructor
-    $resolver = Net::DNS::Resolver->new(
-	nameservers => [ '2001:DB8::1', 'ns.example.com' ],
-	recurse	    => 0,
-	debug	    => 1
-	);
+ # Set options in the constructor
+ $resolver = Net::DNS::Resolver->new(
+			nameservers => [ '2001:DB8::1', 'ns.example.com' ],
+			recurse	    => 0,
+			debug	    => 1
+			);
 
 Returns a resolver object.  If no arguments are supplied, C<new()>
 returns an object having the default configuration.
@@ -88,19 +84,19 @@ The following keywords are recognised in resolver configuration files:
 
 =over
 
-B<nameserver> address
+=item B<nameserver> address
 
 IP address of a name server that the resolver should query.
 
-B<domain> localdomain
+=item B<domain> localdomain
 
 The domain suffix to be appended to a short non-absolute name.
 
-B<search> domain ...
+=item B<search> domain ...
 
 A space-separated list of domains in the desired search path.
 
-B<options> option:value ...
+=item B<options> option:value ...
 
 A space-separated list of key:value items.
 
@@ -118,23 +114,9 @@ On Windows systems, an attempt is made to determine the system defaults
 using the registry.  Systems with many dynamically configured network
 interfaces may confuse L<Net::DNS>.
 
-
-    # Use my own configuration file
-    $resolver = Net::DNS::Resolver->new( config_file => '/my/dns.conf' );
-
-You can include a configuration file of your own when creating a
-resolver object.  This is supported on both Unix and Windows.
-
 If a custom configuration file is specified at first instantiation,
 all other configuration files and environment variables are ignored.
 
-
-    # Set options in the constructor
-    $resolver = Net::DNS::Resolver->new(
-	nameservers => [ '2001:DB8::1', 'ns.example.com' ],
-	recurse	    => 0
-	);
-
 Explicit arguments to C<new()> override the corresponding configuration
 variables.  The argument list consists of a sequence of (name=>value)
 pairs, each interpreted as an invocation of the corresponding method.
@@ -142,18 +124,18 @@ pairs, each interpreted as an invocation of the corresponding method.
 
 =head2 print
 
-    $resolver->print;
+ $resolver->print;
 
 Prints the resolver state on the standard output.
 
 
 =head2 query
 
-    $packet = $resolver->query( 'mailhost' );
-    $packet = $resolver->query( 'mailhost.example.com' );
-    $packet = $resolver->query( '2001:DB8::1' );
-    $packet = $resolver->query( 'example.com', 'MX' );
-    $packet = $resolver->query( 'annotation.example.com', 'TXT', 'IN' );
+ $packet = $resolver->query( 'host' );
+ $packet = $resolver->query( 'host.example.com' );
+ $packet = $resolver->query( '2001:DB8::1' );
+ $packet = $resolver->query( 'example.com', 'MX' );
+ $packet = $resolver->query( 'annotation.example.com', 'TXT', 'IN' );
 
 Performs a DNS query for the given name; the search list is not applied.
 If C<defnames> is true, the default domain will be appended to unqualified names.
@@ -171,11 +153,11 @@ any answers or not, use the C<send()> method instead.
 
 =head2 search
 
-    $packet = $resolver->search( 'mailhost' );
-    $packet = $resolver->search( 'mailhost.example.com' );
-    $packet = $resolver->search( '2001:DB8::1' );
-    $packet = $resolver->search( 'example.com', 'MX' );
-    $packet = $resolver->search( 'annotation.example.com', 'TXT', 'IN' );
+ $packet = $resolver->search( 'host' );
+ $packet = $resolver->search( 'host.example.com' );
+ $packet = $resolver->search( '2001:DB8::1' );
+ $packet = $resolver->search( 'example.com', 'MX' );
+ $packet = $resolver->search( 'annotation.example.com', 'TXT', 'IN' );
 
 Performs a DNS query for the given name, applying the searchlist if
 appropriate.  The search algorithm is as follows:
@@ -199,12 +181,12 @@ any answers or not, use the C<send()> method instead.
 
 =head2 send
 
-    $packet = $resolver->send( $query );
+ $packet = $resolver->send( $query );
 
-    $packet = $resolver->send( 'mailhost.example.com' );
-    $packet = $resolver->send( '2001:DB8::1' );
-    $packet = $resolver->send( 'example.com', 'MX' );
-    $packet = $resolver->send( 'annotation.example.com', 'TXT', 'IN' );
+ $packet = $resolver->send( 'host.example.com' );
+ $packet = $resolver->send( '2001:DB8::1' );
+ $packet = $resolver->send( 'example.com', 'MX' );
+ $packet = $resolver->send( 'annotation.example.com', 'TXT', 'IN' );
 
 Performs a DNS query for the given name.
 Neither the searchlist nor the default domain will be appended.
@@ -223,15 +205,15 @@ Returns C<undef> if no response was received.
 
 =head2 axfr
 
-    @zone = $resolver->axfr();
-    @zone = $resolver->axfr( 'example.com' );
-    @zone = $resolver->axfr( 'example.com', 'IN' );
+ @zone = $resolver->axfr();
+ @zone = $resolver->axfr( 'example.com' );
+ @zone = $resolver->axfr( 'example.com', 'IN' );
 
-    $iterator = $resolver->axfr();
-    $iterator = $resolver->axfr( 'example.com' );
-    $iterator = $resolver->axfr( 'example.com', 'IN' );
+ $iterator = $resolver->axfr();
+ $iterator = $resolver->axfr( 'example.com' );
+ $iterator = $resolver->axfr( 'example.com', 'IN' );
 
-    $rr = $iterator->();
+ $rr = $iterator->();
 
 Performs a zone transfer using the resolver nameservers list,
 attempted in the order listed.
@@ -253,13 +235,13 @@ the reason for failure may be determined using C<errorstring()>.
 
 Here is an example that uses a timeout and TSIG verification:
 
-    $resolver->tcp_timeout( 10 );
-    $resolver->tsig( 'Khmac-sha1.example.+161+24053.private' );
-    @zone = $resolver->axfr( 'example.com' );
+ $resolver->tcp_timeout( 10 );
+ $resolver->tsig( $keyfile );
+ @zone = $resolver->axfr( 'example.com' );
 
-    foreach $rr (@zone) {
-	$rr->print;
-    }
+ foreach $rr (@zone) {
+		$rr->print;
+ }
 
 
 When called in scalar context, C<axfr()> returns an iterator object.
@@ -273,23 +255,23 @@ returned to the caller.
 
 Here is the example above, implemented using an iterator:
 
-    $resolver->tcp_timeout( 10 );
-    $resolver->tsig( 'Khmac-sha1.example.+161+24053.private' );
-    $iterator = $resolver->axfr( 'example.com' );
+ $resolver->tcp_timeout( 10 );
+ $resolver->tsig( $keyfile );
+ $iterator = $resolver->axfr( 'example.com' );
 
-    while ( $rr = $iterator->() ) {
-	$rr->print;
-    }
+ while ( $rr = $iterator->() ) {
+		$rr->print;
+ }
 
 
 =head2 bgsend
 
-    $handle = $resolver->bgsend( $packet ) || die $resolver->errorstring;
+ $handle = $resolver->bgsend( $packet ) || die $resolver->errorstring;
 
-    $handle = $resolver->bgsend( 'mailhost.example.com' );
-    $handle = $resolver->bgsend( '2001:DB8::1' );
-    $handle = $resolver->bgsend( 'example.com', 'MX' );
-    $handle = $resolver->bgsend( 'annotation.example.com', 'TXT', 'IN' );
+ $handle = $resolver->bgsend( 'host.example.com' );
+ $handle = $resolver->bgsend( '2001:DB8::1' );
+ $handle = $resolver->bgsend( 'example.com', 'MX' );
+ $handle = $resolver->bgsend( 'annotation.example.com', 'TXT', 'IN' );
 
 Performs a background DNS query for the given name and returns immediately
 without waiting for the response. The program can then perform other tasks
@@ -314,8 +296,8 @@ returned by C<bgsend()> which should be used strictly as described here.
 
 =head2 bgread
 
-    $handle = $resolver->bgsend( 'www.example.com' );
-    $packet = $resolver->bgread($handle);
+ $handle = $resolver->bgsend( 'www.example.com' );
+ $packet = $resolver->bgread($handle);
 
 Reads the response following a background query.
 The argument is the handle returned by C<bgsend()>.
@@ -326,13 +308,13 @@ received before the timeout interval expired.
 
 =head2 bgbusy
 
-    $handle = $resolver->bgsend( 'foo.example.com' );
+ $handle = $resolver->bgsend( 'foo.example.com' );
 
-    while ($resolver->bgbusy($handle)) {
-	...
-    }
+ while ($resolver->bgbusy($handle)) {
+		...
+ }
 
-    $packet = $resolver->bgread($handle);
+ $packet = $resolver->bgread($handle);
 
 Returns true while awaiting the response or for the transaction to time out.
 The argument is the handle returned by C<bgsend()>.
@@ -343,8 +325,8 @@ continuing to assert busy to the caller.
 
 =head2 debug
 
-    print 'debug flag: ', $resolver->debug, "\n";
-    $resolver->debug(1);
+ print 'debug flag: ', $resolver->debug, "\n";
+ $resolver->debug(1);
 
 Get or set the debug flag.
 If set, calls to C<search()>, C<query()>, and C<send()> will print
@@ -354,8 +336,8 @@ The default is false.
 
 =head2 defnames
 
-    print 'defnames flag: ', $resolver->defnames, "\n";
-    $resolver->defnames(0);
+ print 'defnames flag: ', $resolver->defnames, "\n";
+ $resolver->defnames(0);
 
 Get or set the defnames flag.
 If true, calls to C<query()> will append the default domain to
@@ -365,8 +347,8 @@ The default is true.
 
 =head2 dnsrch
 
-    print 'dnsrch flag: ', $resolver->dnsrch, "\n";
-    $resolver->dnsrch(0);
+ print 'dnsrch flag: ', $resolver->dnsrch, "\n";
+ $resolver->dnsrch(0);
 
 Get or set the dnsrch flag.
 If true, calls to C<search()> will apply the search list to resolve
@@ -376,16 +358,16 @@ The default is true.
 
 =head2 domain
 
-    $domain = $resolver->domain;
-    $resolver->domain( 'domain.example' );
+ $domain = $resolver->domain;
+ $resolver->domain( 'domain.example' );
 
 Gets or sets the resolver default domain.
 
 
 =head2 igntc
 
-    print 'igntc flag: ', $resolver->igntc, "\n";
-    $resolver->igntc(1);
+ print 'igntc flag: ', $resolver->igntc, "\n";
+ $resolver->igntc(1);
 
 Get or set the igntc flag.
 If true, truncated packets will be ignored.
@@ -395,9 +377,9 @@ The default is false.
 
 =head2 nameserver, nameservers
 
-    @nameservers = $resolver->nameservers();
-    $resolver->nameservers( '2001:DB8::1', '192.0.2.1' );
-    $resolver->nameservers( 'ns.domain.example.' );
+ @nameservers = $resolver->nameservers();
+ $resolver->nameservers( '2001:DB8::1', '192.0.2.1' );
+ $resolver->nameservers( 'ns.domain.example.' );
 
 Gets or sets the nameservers to be queried.
 
@@ -406,8 +388,8 @@ Also see the IPv6 transport notes below
 
 =head2 persistent_tcp
 
-    print 'Persistent TCP flag: ', $resolver->persistent_tcp, "\n";
-    $resolver->persistent_tcp(1);
+ print 'Persistent TCP flag: ', $resolver->persistent_tcp, "\n";
+ $resolver->persistent_tcp(1);
 
 Get or set the persistent TCP setting.
 If true, L<Net::DNS> will keep a TCP socket open for each host:port
@@ -421,8 +403,8 @@ in which case the default is true.
 
 =head2 persistent_udp
 
-    print 'Persistent UDP flag: ', $resolver->persistent_udp, "\n";
-    $resolver->persistent_udp(1);
+ print 'Persistent UDP flag: ', $resolver->persistent_udp, "\n";
+ $resolver->persistent_udp(1);
 
 Get or set the persistent UDP setting.
 If true, a L<Net::DNS> resolver will use the same UDP socket
@@ -434,8 +416,8 @@ but also defeats source port randomisation.
 
 =head2 port
 
-    print 'sending queries to port ', $resolver->port, "\n";
-    $resolver->port(9732);
+ print 'sending queries to port ', $resolver->port, "\n";
+ $resolver->port(9732);
 
 Gets or sets the port to which queries are sent.
 Convenient for nameserver testing using a non-standard port.
@@ -444,8 +426,8 @@ The default is port 53.
 
 =head2 recurse
 
-    print 'recursion flag: ', $resolver->recurse, "\n";
-    $resolver->recurse(0);
+ print 'recursion flag: ', $resolver->recurse, "\n";
+ $resolver->recurse(0);
 
 Get or set the recursion flag.
 If true, this will direct nameservers to perform a recursive query.
@@ -454,8 +436,8 @@ The default is true.
 
 =head2 retrans
 
-    print 'retrans interval: ', $resolver->retrans, "\n";
-    $resolver->retrans(3);
+ print 'retrans interval: ', $resolver->retrans, "\n";
+ $resolver->retrans(3);
 
 Get or set the retransmission interval
 The default is 5 seconds.
@@ -463,8 +445,8 @@ The default is 5 seconds.
 
 =head2 retry
 
-    print 'number of tries: ', $resolver->retry, "\n";
-    $resolver->retry(2);
+ print 'number of tries: ', $resolver->retry, "\n";
+ $resolver->retry(2);
 
 Get or set the number of times to try the query.
 The default is 4.
@@ -472,15 +454,15 @@ The default is 4.
 
 =head2 searchlist
 
-    @searchlist = $resolver->searchlist;
-    $resolver->searchlist( 'a.example', 'b.example', 'c.example' );
+ @searchlist = $resolver->searchlist;
+ $resolver->searchlist( 'a.example', 'b.example', 'c.example' );
 
 Gets or sets the resolver search list.
 
 
 =head2 srcaddr
 
-    $resolver->srcaddr('2001::DB8::1');
+ $resolver->srcaddr('2001::DB8::1');
 
 Sets the source address from which queries are sent.
 Convenient for forcing queries from a specific interface on a
@@ -489,7 +471,7 @@ multi-homed host.  The default is to use any local address.
 
 =head2 srcport
 
-    $resolver->srcport(5353);
+ $resolver->srcport(5353);
 
 Sets the port from which queries are sent.
 The default is 0, meaning any port.
@@ -497,8 +479,8 @@ The default is 0, meaning any port.
 
 =head2 tcp_timeout
 
-    print 'TCP timeout: ', $resolver->tcp_timeout, "\n";
-    $resolver->tcp_timeout(10);
+ print 'TCP timeout: ', $resolver->tcp_timeout, "\n";
+ $resolver->tcp_timeout(10);
 
 Get or set the TCP timeout in seconds.
 The default is 120 seconds (2 minutes).
@@ -506,8 +488,8 @@ The default is 120 seconds (2 minutes).
 
 =head2 udp_timeout
 
-    print 'UDP timeout: ', $resolver->udp_timeout, "\n";
-    $resolver->udp_timeout(10);
+ print 'UDP timeout: ', $resolver->udp_timeout, "\n";
+ $resolver->udp_timeout(10);
 
 Get or set the bgsend() UDP timeout in seconds.
 The default is 30 seconds.
@@ -515,8 +497,8 @@ The default is 30 seconds.
 
 =head2 udppacketsize
 
-    print "udppacketsize: ", $resolver->udppacketsize, "\n";
-    $resolver->udppacketsize(2048);
+ print "udppacketsize: ", $resolver->udppacketsize, "\n";
+ $resolver->udppacketsize(2048);
 
 Get or set the UDP packet size.
 If set to a value not less than the default DNS packet size,
@@ -526,8 +508,8 @@ large UDP datagrams.
 
 =head2 usevc
 
-    print 'usevc flag: ', $resolver->usevc, "\n";
-    $resolver->usevc(1);
+ print 'usevc flag: ', $resolver->usevc, "\n";
+ $resolver->usevc(1);
 
 Get or set the usevc flag.
 If true, queries will be performed using virtual circuits (TCP)
@@ -537,7 +519,7 @@ The default is false.
 
 =head2 replyfrom
 
-    print 'last response was from: ', $resolver->replyfrom, "\n";
+ print 'last response was from: ', $resolver->replyfrom, "\n";
 
 Returns the IP address from which the most recent packet was
 received in response to a query.
@@ -545,7 +527,7 @@ received in response to a query.
 
 =head2 errorstring
 
-    print 'query status: ', $resolver->errorstring, "\n";
+ print 'query status: ', $resolver->errorstring, "\n";
 
 Returns a string containing error information from the most recent
 DNS protocol interaction.
@@ -555,8 +537,8 @@ after the corresponding method call.
 
 =head2 dnssec
 
-    print "dnssec flag: ", $resolver->dnssec, "\n";
-    $resolver->dnssec(0);
+ print "dnssec flag: ", $resolver->dnssec, "\n";
+ $resolver->dnssec(0);
 
 The dnssec flag causes the resolver to transmit DNSSEC queries
 and to add a EDNS0 record as required by RFC2671 and RFC3225.
@@ -568,8 +550,8 @@ UDP packet size to the default value of 2048. If that is too small or
 too big for your environment, you should call the C<udppacketsize()>
 method immediately after.
 
-   $resolver->dnssec(1);		# DNSSEC using default packetsize
-   $resolver->udppacketsize(1250);	# lower the UDP packet size
+ $resolver->dnssec(1);
+ $resolver->udppacketsize(1250);	# adjust UDP packet size
 
 A fatal exception will be raised if the C<dnssec()> method is called
 but the L<Net::DNS::SEC> library has not been installed.
@@ -577,9 +559,9 @@ but the L<Net::DNS::SEC> library has not been installed.
 
 =head2 adflag
 
-    $resolver->dnssec(1);
-    $resolver->adflag(1);
-    print "authentication desired flag: ", $resolver->adflag, "\n";
+ $resolver->dnssec(1);
+ $resolver->adflag(1);
+ print "authentication desired flag: ", $resolver->adflag, "\n";
 
 Gets or sets the AD bit for dnssec queries.  This bit indicates that
 the caller is interested in the returned AD (authentic data) bit but
@@ -589,9 +571,9 @@ The default value is false.
 
 =head2 cdflag
 
-    $resolver->dnssec(1);
-    $resolver->cdflag(1);
-    print "checking disabled flag: ", $resolver->cdflag, "\n";
+ $resolver->dnssec(1);
+ $resolver->cdflag(1);
+ print "checking disabled flag: ", $resolver->cdflag, "\n";
 
 Gets or sets the CD bit for dnssec queries.  This bit indicates that
 authentication by upstream nameservers should be suppressed.
@@ -602,19 +584,13 @@ The default value is false.
 
 =head2 tsig
 
-    $resolver->tsig( $tsig );
+ $resolver->tsig( $keyfile );
 
-    $resolver->tsig( 'Khmac-sha1.example.+161+24053.private' );
+ $resolver->tsig( $keyfile, fudge => 60 );
 
-    $resolver->tsig( 'Khmac-sha1.example.+161+24053.key' );
+ $resolver->tsig( $tsig_rr );
 
-    $resolver->tsig( 'Khmac-sha1.example.+161+24053.key',
-		fudge => 60
-		);
-
-    $resolver->tsig( $key_name, $key );
-
-    $resolver->tsig( undef );
+ $resolver->tsig( undef );
 
 Set the TSIG record used to automatically sign outgoing queries, zone
 transfers and updates. Automatic signing is disabled if called with
@@ -637,45 +613,45 @@ the resolver:
 
 =head2 RES_NAMESERVERS
 
-    # Bourne Shell
-    RES_NAMESERVERS="2001:DB8::1 192.0.2.1"
-    export RES_NAMESERVERS
+ # Bourne Shell
+ RES_NAMESERVERS="2001:DB8::1 192.0.2.1"
+ export RES_NAMESERVERS
 
-    # C Shell
-    setenv RES_NAMESERVERS "2001:DB8::1 192.0.2.1"
+ # C Shell
+ setenv RES_NAMESERVERS "2001:DB8::1 192.0.2.1"
 
 A space-separated list of nameservers to query.
 
 =head2 RES_SEARCHLIST
 
-    # Bourne Shell
-    RES_SEARCHLIST="a.example.com b.example.com c.example.com"
-    export RES_SEARCHLIST
+ # Bourne Shell
+ RES_SEARCHLIST="a.example.com b.example.com c.example.com"
+ export RES_SEARCHLIST
 
-    # C Shell
-    setenv RES_SEARCHLIST "a.example.com b.example.com c.example.com"
+ # C Shell
+ setenv RES_SEARCHLIST "a.example.com b.example.com c.example.com"
 
 A space-separated list of domains to put in the search list.
 
 =head2 LOCALDOMAIN
 
-    # Bourne Shell
-    LOCALDOMAIN=example.com
-    export LOCALDOMAIN
+ # Bourne Shell
+ LOCALDOMAIN=example.com
+ export LOCALDOMAIN
 
-    # C Shell
-    setenv LOCALDOMAIN example.com
+ # C Shell
+ setenv LOCALDOMAIN example.com
 
 The default domain.
 
 =head2 RES_OPTIONS
 
-    # Bourne Shell
-    RES_OPTIONS="retrans:3 retry:2 inet6"
-    export RES_OPTIONS
+ # Bourne Shell
+ RES_OPTIONS="retrans:3 retry:2 inet6"
+ export RES_OPTIONS
 
-    # C Shell
-    setenv RES_OPTIONS "retrans:3 retry:2 inet6"
+ # C Shell
+ setenv RES_OPTIONS "retrans:3 retry:2 inet6"
 
 A space-separated list of resolver options to set.  Options that
 take values are specified as C<option:value>.
@@ -690,13 +666,13 @@ The behaviour of the C<nameserver()> method illustrates the transport
 selection mechanism.  If, for example, IPv4 transport has been forced,
 the C<nameserver()> method will only return IPv4 addresses:
 
-    $resolver->nameservers( '192.0.2.1', '192.0.2.2', '2001:DB8::3' );
-    $resolver->force_v4(1);
-    print join ' ', $resolver->nameservers();
+ $resolver->nameservers( '192.0.2.1', '192.0.2.2', '2001:DB8::3' );
+ $resolver->force_v4(1);
+ print join ' ', $resolver->nameservers();
 
 will print
 
-    192.0.2.1 192.0.2.2
+ 192.0.2.1 192.0.2.2
 
 
 =head1 CUSTOMISED RESOLVERS
@@ -708,15 +684,14 @@ You can simply add a method to the namespace!
 
 For example, if we wanted to cache lookups:
 
-    package Net::DNS::Resolver;
+ package Net::DNS::Resolver;
 
-    my %cache;
+ my %cache;
 
-    sub search {
-	$self = shift;
-
-	$cache{"@_"} ||= $self->SUPER::search(@_);
-    }
+ sub send {
+     $self = shift;
+     $cache{"@_"} ||= $self->SUPER::send(@_);
+ }
 
 
 =head1 COPYRIGHT
@@ -736,7 +711,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -753,9 +728,11 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::Packet>, L<Net::DNS::Update>,
-L<Net::DNS::Header>, L<Net::DNS::Question>, L<Net::DNS::RR>,
-L<resolver(5)>, RFC 1034, RFC 1035
+L<perl> L<Net::DNS> L<Net::DNS::Packet> L<Net::DNS::Header>
+L<Net::DNS::Question> L<Net::DNS::RR>
+L<resolver(5)>
+L<RFC1034|https://iana.org/go/rfc1034>
+L<RFC1035|https://iana.org/go/rfc1035>
 
 =cut
 
