@@ -2,7 +2,7 @@ package Net::DNS::Resolver::Base;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: Base.pm 1945 2023-11-22 08:02:31Z willem $)[2];
+our $VERSION = (qw$Id: Base.pm 1996 2024-12-16 13:05:08Z willem $)[2];
 
 
 #
@@ -25,7 +25,12 @@ our $VERSION = (qw$Id: Base.pm 1945 2023-11-22 08:02:31Z willem $)[2];
 # [Revised March 2016, June 2018]
 
 
-use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.38; 1;';    ## no critic
+use constant OS_SPEC => defined eval "require Net::DNS::Resolver::$^O";	## no critic
+use constant OS_CONF => join '::', 'Net::DNS::Resolver', OS_SPEC ? $^O : 'UNIX';
+use base OS_CONF;
+
+
+use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.38; 1;';	## no critic
 require IO::Socket::INET unless USE_SOCKET_IP;
 
 use constant IPv6 => USE_SOCKET_IP;
@@ -45,7 +50,13 @@ use Carp;
 use IO::File;
 use IO::Select;
 use IO::Socket;
+
 use Socket;
+{
+	no strict 'subs';		## no critic ProhibitNoStrict
+	use constant AI_NUMERICHOST => Socket::AI_NUMERICHOST;
+	use constant IPPROTO_UDP    => Socket::IPPROTO_UDP;
+}
 
 use Net::DNS::RR;
 use Net::DNS::Packet;
@@ -407,7 +418,7 @@ sub search {
 sub send {
 	my ( $self, @argument ) = @_;
 	my $packet	= $self->_make_query_packet(@argument);
-	my $packet_data = $packet->data;
+	my $packet_data = $packet->encode;
 
 	$self->_reset_errorstring;
 
@@ -442,14 +453,16 @@ sub _send_tcp {
 		$socket->send($tcp_packet);
 		$self->errorstring($!);
 
-		my $buffer = _read_tcp($socket);
-		$self->{replyfrom} = $ip;
-		$self->_diag( 'reply from', "[$ip]", length($buffer), 'bytes' );
+		my ($peer) = $select->can_read($timeout);
+		next unless $peer;				# uncoverable branch true
+		my $buffer = _read_tcp($peer);
+		my $host   = $self->{replyfrom} = $peer->peerhost;
+		$self->_diag( 'packet from', "[$host]", length($buffer), 'octets' );
 
 		my $reply = Net::DNS::Packet->decode( \$buffer, $self->{debug} );
 		$self->errorstring($@);
 		next unless $self->_accept_reply( $reply, $query );
-		$reply->from($ip);
+		$reply->from($host);
 
 		if ( $self->{tsig_rr} && !$reply->verify($query) ) {
 			$self->errorstring( $reply->verifyerr );
@@ -509,10 +522,10 @@ NAMESERVER: foreach my $ns (@ns) {
 
 			my $reply;
 			while ( my ($socket) = $select->can_read($timeout) ) {
-				my $peer = $self->{replyfrom} = $socket->peerhost;
-
 				my $buffer = _read_udp($socket);
-				$self->_diag( "reply from [$peer]", length($buffer), 'bytes' );
+
+				my $peer = $self->{replyfrom} = $socket->peerhost;
+				$self->_diag( "packet from [$peer]", length($buffer), 'octets' );
 
 				my $packet = Net::DNS::Packet->decode( \$buffer, $self->{debug} );
 				$self->errorstring($@);
@@ -549,7 +562,7 @@ NAMESERVER: foreach my $ns (@ns) {
 sub bgsend {
 	my ( $self, @argument ) = @_;
 	my $packet	= $self->_make_query_packet(@argument);
-	my $packet_data = $packet->data;
+	my $packet_data = $packet->encode;
 
 	$self->_reset_errorstring;
 
@@ -621,7 +634,7 @@ sub bgbusy {				## no critic		# overwrites user UDP handle
 	my ( $expire, $query, $read ) = @$appendix;
 	return if ref($read);
 
-	return time() < $expire unless IO::Select->new($handle)->can_read(0);
+	return time() < $expire unless IO::Select->new($handle)->can_read(0.02);    # limit CPU burn
 
 	return unless $query;					# SpamAssassin 3.4.1 workaround
 	return unless $handle->socktype() == SOCK_DGRAM;
@@ -634,7 +647,7 @@ sub bgbusy {				## no critic		# overwrites user UDP handle
 	return unless $ans->header->tc;
 
 	$self->_diag('packet truncated: retrying using TCP');
-	my $tcp = $self->_bgsend_tcp( $query, $query->data ) || return;
+	my $tcp = $self->_bgsend_tcp( $query, $query->encode ) || return;
 	return defined( $_[1] = $tcp );				# caller's UDP handle now TCP
 }
 
@@ -646,10 +659,7 @@ sub bgisready {				## historical
 
 
 sub bgread {
-	my ( $self, $handle ) = @_;
-	while (&bgbusy) {					# side effect: TCP retry
-		IO::Select->new($handle)->can_read(0.02);	# reduce my CPU usage by 3 orders of magnitude
-	}
+	1 while &bgbusy;		## side effect: TCP retry if TC flag set
 	return &_bgread;
 }
 
@@ -662,21 +672,19 @@ sub _bgread {
 	my ( $expire, $query, $read ) = @$appendix;
 	return shift(@$read) if ref($read);
 
-	my $select = IO::Select->new($handle);
-	unless ( $select->can_read(0) ) {
-		$self->errorstring('timed out');
-		return;
-	}
+	return unless IO::Select->new($handle)->can_read(0.2);
 
 	my $dgram  = $handle->socktype() == SOCK_DGRAM;
 	my $buffer = $dgram ? _read_udp($handle) : _read_tcp($handle);
-	$self->_diag( 'read', length($buffer), 'bytes' );
+
+	my $peer = $self->{replyfrom} = $handle->peerhost;
+	$self->_diag( "packet from [$peer]", length($buffer), 'octets' );
 
 	my $reply = Net::DNS::Packet->decode( \$buffer, $self->{debug} );
 	$self->errorstring($@);
 
 	return unless $self->_accept_reply( $reply, $query );
-	$reply->from( $handle->peerhost );
+	$reply->from($peer);
 
 	return $reply unless $self->{tsig_rr} && !$reply->verify($query);
 	$self->errorstring( $reply->verifyerr );
@@ -760,7 +768,7 @@ sub axfr_next {				## historical
 
 sub _axfr_start {
 	my ( $self, $request ) = @_;
-	my $content = $request->data;
+	my $content = $request->encode;
 	my $TCP_msg = pack 'n a*', length($content), $content;
 
 	my ( $select, $reply, $rcode );
@@ -819,19 +827,24 @@ sub _axfr_next {
 #
 # Usage:  $data = _read_tcp($socket);
 #
+sub _read_socket {
+	my ( $socket, $size ) = @_;
+	my $buffer = '';
+	$socket->recv( $buffer, $size ) if $size;
+	return $buffer;
+}
+
 sub _read_tcp {
 	my $socket = shift;
 
-	my ( $s1, $s2 );
-	$socket->recv( $s1, 1 );				# two octet length
-	$socket->recv( $s2, 2 - length $s1 );			# possibly fragmented
-	my $size = unpack 'n', pack( 'a*a*@2', $s1, $s2 );
-
 	my $buffer = '';
-	for ( ; ; ) {
-		my $fragment;
-		$socket->recv( $fragment, $size - length($buffer) );
-		last unless length( $buffer .= $fragment || last ) < $size;
+	my $header = _read_socket( $socket, 2 );
+	$header .= _read_socket( $socket, 2 - length $header );
+	return $buffer if length($header) < 2;			# uncoverable branch true
+	my $size = unpack 'n', $header;
+
+	while ( my $fragment = _read_socket( $socket, $size - length $buffer ) ) {
+		$buffer .= $fragment;
 	}
 	return $buffer;
 }
@@ -841,10 +854,7 @@ sub _read_tcp {
 # Usage:  $data = _read_udp($socket);
 #
 sub _read_udp {
-	my $socket = shift;
-	my $buffer = '';
-	$socket->recv( $buffer, 9000 );	## payload limit for Ethernet "Jumbo" packet
-	return $buffer;
+	return _read_socket( shift(), 9000 );	## payload limit for Ethernet "Jumbo" packet
 }
 
 
@@ -922,34 +932,28 @@ sub _create_udp_socket {
 }
 
 
-{
-	no strict 'subs';		## no critic ProhibitNoStrict
-	use constant AI_NUMERICHOST => Socket::AI_NUMERICHOST;
-	use constant IPPROTO_UDP    => Socket::IPPROTO_UDP;
+my $ip4 = {
+	family	 => AF_INET,
+	flags	 => AI_NUMERICHOST,
+	protocol => IPPROTO_UDP,
+	socktype => SOCK_DGRAM
+	};
+my $ip6 = {
+	family	 => AF_INET6,
+	flags	 => AI_NUMERICHOST,
+	protocol => IPPROTO_UDP,
+	socktype => SOCK_DGRAM
+	};
 
-	my $ip4 = {
-		family	 => AF_INET,
-		flags	 => AI_NUMERICHOST,
-		protocol => IPPROTO_UDP,
-		socktype => SOCK_DGRAM
-		};
-	my $ip6 = {
-		family	 => AF_INET6,
-		flags	 => AI_NUMERICHOST,
-		protocol => IPPROTO_UDP,
-		socktype => SOCK_DGRAM
-		};
+sub _create_dst_sockaddr {		## create UDP destination sockaddr structure
+	my ( $self, $ip, $port ) = @_;
 
-	sub _create_dst_sockaddr {	## create UDP destination sockaddr structure
-		my ( $self, $ip, $port ) = @_;
-
-		unless (USE_SOCKET_IP) {			# NB: errors raised in socket->send
-			return _ipv6($ip) ? undef : sockaddr_in( $port, inet_aton($ip) );
-		}
-
-		my @addrinfo = Socket::getaddrinfo( $ip, $port, _ipv6($ip) ? $ip6 : $ip4 );
-		return ( grep {ref} @addrinfo, {} )[0]->{addr};
+	unless (USE_SOCKET_IP) {				# NB: errors raised in socket->send
+		return _ipv6($ip) ? undef : sockaddr_in( $port, inet_aton($ip) );
 	}
+
+	my @addrinfo = Socket::getaddrinfo( $ip, $port, _ipv6($ip) ? $ip6 : $ip4 );
+	return ( grep {ref} @addrinfo, {} )[0]->{addr};
 }
 
 
@@ -978,10 +982,7 @@ sub _make_query_packet {
 	my ( $self, @argument ) = @_;
 
 	my ($packet) = @argument;
-	if ( ref($packet) ) {
-		my $edns = $packet->edns;			# advertise UDPsize for local stack
-		$edns->udpsize( $self->{udppacketsize} ) unless defined $edns->{udpsize};
-	} else {
+	unless ( ref($packet) ) {
 		$packet = Net::DNS::Packet->new(@argument);
 		$packet->edns->udpsize( $self->{udppacketsize} );
 
@@ -1213,17 +1214,16 @@ L<perl> L<Net::DNS> L<Net::DNS::Resolver>
 
 __DATA__	## DEFAULT HINTS
 
-; <<>> DiG 9.18.8 <<>> @b.root-servers.net . -t NS
+; <<>> DiG 9.18.20 <<>> @b.root-servers.net . -t NS
 ; (2 servers found)
 ;; global options: +cmd
 ;; Got answer:
-;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 4847
+;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 938
 ;; flags: qr aa rd; QUERY: 1, ANSWER: 13, AUTHORITY: 0, ADDITIONAL: 27
 ;; WARNING: recursion requested but not available
 
 ;; OPT PSEUDOSECTION:
 ; EDNS: version: 0, flags:; udp: 1232
-; COOKIE: 93d86b753941ac2f0100000063c009ad51783c108e36cf16 (good)
 ;; QUESTION SECTION:
 ;.				IN	NS
 
@@ -1245,8 +1245,8 @@ __DATA__	## DEFAULT HINTS
 ;; ADDITIONAL SECTION:
 a.root-servers.net.	518400	IN	A	198.41.0.4
 a.root-servers.net.	518400	IN	AAAA	2001:503:ba3e::2:30
-b.root-servers.net.	518400	IN	A	199.9.14.201
-b.root-servers.net.	518400	IN	AAAA	2001:500:200::b
+b.root-servers.net.	518400	IN	A	170.247.170.2
+b.root-servers.net.	518400	IN	AAAA	2801:1b8:10::b
 c.root-servers.net.	518400	IN	A	192.33.4.12
 c.root-servers.net.	518400	IN	AAAA	2001:500:2::c
 d.root-servers.net.	518400	IN	A	199.7.91.13
@@ -1270,8 +1270,8 @@ l.root-servers.net.	518400	IN	AAAA	2001:500:9f::42
 m.root-servers.net.	518400	IN	A	202.12.27.33
 m.root-servers.net.	518400	IN	AAAA	2001:dc3::35
 
-;; Query time: 15 msec
-;; SERVER: 199.9.14.201#53(b.root-servers.net) (UDP)
-;; WHEN: Thu Jan 12 13:22:53 GMT 2023
-;; MSG SIZE  rcvd: 1031
+;; Query time: 19 msec
+;; SERVER: 170.247.170.2#53(b.root-servers.net) (UDP)
+;; WHEN: Fri Dec 29 22:01:37 GMT 2023
+;; MSG SIZE  rcvd: 1003
 
