@@ -3,7 +3,7 @@ package Net::DNS::ZoneFile;
 use strict;
 use warnings;
 
-our $VERSION = (qw$Id: ZoneFile.pm 1847 2021-08-11 10:02:44Z willem $)[2];
+our $VERSION = (qw$Id: ZoneFile.pm 2000 2025-01-02 16:34:13Z willem $)[2];
 
 
 =head1 NAME
@@ -12,15 +12,15 @@ Net::DNS::ZoneFile - DNS zone file
 
 =head1 SYNOPSIS
 
-    use Net::DNS::ZoneFile;
+ use Net::DNS::ZoneFile;
 
-    $zonefile = Net::DNS::ZoneFile->new( 'named.example' );
+ $zonefile = Net::DNS::ZoneFile->new( 'named.example' );
 
-    while ( $rr = $zonefile->read ) {
-	$rr->print;
-    }
+ while ( $rr = $zonefile->read ) {
+		$rr->print;
+ }
 
-    @zone = $zonefile->read;
+ @zone = $zonefile->read;
 
 
 =head1 DESCRIPTION
@@ -44,18 +44,17 @@ automatically to all subsequent records.
 
 use integer;
 use Carp;
-use IO::File;
 
 use base qw(Exporter);
 our @EXPORT = qw(parse read readfh);
 
-use constant PERLIO => defined eval { require PerlIO };
-
 use constant UTF8 => scalar eval {	## not UTF-EBCDIC  [see Unicode TR#16 3.6]
 	require Encode;
 	Encode::encode_utf8( chr(182) ) eq pack( 'H*', 'C2B6' );
 };
 
+require IO::File;
+require PerlIO;
 require Net::DNS::Domain;
 require Net::DNS::RR;
 
@@ -65,10 +64,10 @@ require Net::DNS::RR;
 
 =head2 new
 
-    $zonefile = Net::DNS::ZoneFile->new( 'filename', ['example.com'] );
+ $zonefile = Net::DNS::ZoneFile->new( 'filename', ['example.com'] );
 
-    $handle   = IO::File->new( 'filename', '<:encoding(ISO8859-7)' );
-    $zonefile = Net::DNS::ZoneFile->new( $handle, ['example.com'] );
+ $handle   = IO::File->new( 'filename', '<:encoding(ISO8859-7)' );
+ $zonefile = Net::DNS::ZoneFile->new( $handle, ['example.com'] );
 
 The new() constructor returns a Net::DNS::ZoneFile object which
 represents the zone file specified in the argument list.
@@ -88,30 +87,30 @@ introduced by $INCLUDE directives.
 =cut
 
 sub new {
-	my $self = bless {}, shift;
-	my $file = shift;
-	$self->_origin(shift);
+	my ( $class, $filename, $origin ) = @_;
+	my $self = bless {fileopen => {}}, $class;
 
-	if ( ref($file) ) {
-		$self->{filename} = $self->{filehandle} = $file;
-		$self->{fileopen} = {};
-		return $self if ref($file) =~ /IO::File|FileHandle|GLOB|Text/;
+	$self->_origin($origin);
+
+	if ( ref($filename) ) {
+		$self->{filehandle} = $self->{filename} = $filename;
+		return $self if ref($filename) =~ /IO::File|FileHandle|GLOB|Text/;
 		croak 'argument not a file handle';
 	}
 
-	croak 'filename argument undefined' unless $file;
+	croak 'filename argument undefined' unless $filename;
 	my $discipline = UTF8 ? '<:encoding(UTF-8)' : '<';
-	$self->{filehandle} = IO::File->new( $file, $discipline ) or croak "$file: $!";
-	$self->{fileopen}{$file}++;
-	$self->{filename} = $file;
+	$self->{filehandle} = IO::File->new( $filename, $discipline ) or croak "$filename: $!";
+	$self->{fileopen}->{$filename}++;
+	$self->{filename} = $filename;
 	return $self;
 }
 
 
 =head2 read
 
-    $rr = $zonefile->read;
-    @rr = $zonefile->read;
+ $rr = $zonefile->read;
+ @rr = $zonefile->read;
 
 When invoked in scalar context, read() returns a Net::DNS::RR object
 representing the next resource record encountered in the zone file,
@@ -132,19 +131,22 @@ sub read {
 
 	return &_read unless ref $self;				# compatibility interface
 
-	local $SIG{__DIE__};
-
 	if (wantarray) {
 		my @zone;					# return entire zone
 		eval {
-			my $rr;
-			push( @zone, $rr ) while $rr = $self->_getRR;
+			local $SIG{__DIE__};
+			while ( my $rr = $self->_getRR ) {
+				push( @zone, $rr );
+			}
 		};
 		croak join ' ', $@, ' file', $self->name, 'line', $self->line, "\n " if $@;
 		return @zone;
 	}
 
-	my $rr = eval { $self->_getRR };			# return single RR
+	my $rr = eval {
+		local $SIG{__DIE__};
+		$self->_getRR;					# return single RR
+	};
 	croak join ' ', $@, ' file', $self->name, 'line', $self->line, "\n " if $@;
 	return $rr;
 }
@@ -152,7 +154,7 @@ sub read {
 
 =head2 name
 
-    $filename = $zonefile->name;
+ $filename = $zonefile->name;
 
 Returns the name of the current zone file.
 Embedded $INCLUDE directives will cause this to differ from the
@@ -167,7 +169,7 @@ sub name {
 
 =head2 line
 
-    $line = $zonefile->line;
+ $line = $zonefile->line;
 
 Returns the number of the last line read from the current zone file.
 
@@ -182,7 +184,7 @@ sub line {
 
 =head2 origin
 
-    $origin = $zonefile->origin;
+ $origin = $zonefile->origin;
 
 Returns the fully qualified name of the current origin within the
 zone file.
@@ -197,7 +199,7 @@ sub origin {
 
 =head2 ttl
 
-    $ttl = $zonefile->ttl;
+ $ttl = $zonefile->ttl;
 
 Returns the default TTL as specified by the $TTL directive.
 
@@ -215,20 +217,20 @@ CPAN distribution will continue to operate with minimal change using
 the compatibility interface described below.
 New application code should use the object-oriented interface.
 
-    use Net::DNS::ZoneFile;
+ use Net::DNS::ZoneFile;
 
-    $listref = Net::DNS::ZoneFile->read( $filename );
-    $listref = Net::DNS::ZoneFile->read( $filename, $include_dir );
+ $listref = Net::DNS::ZoneFile->read( $filename );
+ $listref = Net::DNS::ZoneFile->read( $filename, $include_dir );
 
-    $listref = Net::DNS::ZoneFile->readfh( $filehandle );
-    $listref = Net::DNS::ZoneFile->readfh( $filehandle, $include_dir );
+ $listref = Net::DNS::ZoneFile->readfh( $filehandle );
+ $listref = Net::DNS::ZoneFile->readfh( $filehandle, $include_dir );
 
-    $listref = Net::DNS::ZoneFile->parse(  $string );
-    $listref = Net::DNS::ZoneFile->parse( \$string );
-    $listref = Net::DNS::ZoneFile->parse(  $string, $include_dir );
-    $listref = Net::DNS::ZoneFile->parse( \$string, $include_dir );
+ $listref = Net::DNS::ZoneFile->parse( $string );
+ $listref = Net::DNS::ZoneFile->parse( $string, $include_dir );
+ $listref = Net::DNS::ZoneFile->parse( \$string );
+ $listref = Net::DNS::ZoneFile->parse( \$string, $include_dir );
 
-    $_->print for @$listref;
+ $_->print for @$listref;
 
 The optional second argument specifies the default path for filenames.
 The current working directory is used by default.
@@ -236,15 +238,15 @@ The current working directory is used by default.
 Although not available in the original implementation, the RR list can
 be obtained directly by calling any of these methods in list context.
 
-    @rr = Net::DNS::ZoneFile->read( $filename, $include_dir );
+ @rr = Net::DNS::ZoneFile->read( $filename, $include_dir );
 
 The partial result is returned if an error is encountered by the parser.
 
 
 =head2 read
 
-    $listref = Net::DNS::ZoneFile->read( $filename );
-    $listref = Net::DNS::ZoneFile->read( $filename, $include_dir );
+ $listref = Net::DNS::ZoneFile->read( $filename );
+ $listref = Net::DNS::ZoneFile->read( $filename, $include_dir );
 
 read() parses the contents of the specified file
 and returns a reference to the list of Net::DNS::RR objects.
@@ -317,8 +319,8 @@ sub _read {
 
 =head2 readfh
 
-    $listref = Net::DNS::ZoneFile->readfh( $filehandle );
-    $listref = Net::DNS::ZoneFile->readfh( $filehandle, $include_dir );
+ $listref = Net::DNS::ZoneFile->readfh( $filehandle );
+ $listref = Net::DNS::ZoneFile->readfh( $filehandle, $include_dir );
 
 readfh() parses data from the specified file handle
 and returns a reference to the list of Net::DNS::RR objects.
@@ -333,10 +335,10 @@ sub readfh {
 
 =head2 parse
 
-    $listref = Net::DNS::ZoneFile->parse(  $string );
-    $listref = Net::DNS::ZoneFile->parse( \$string );
-    $listref = Net::DNS::ZoneFile->parse(  $string, $include_dir );
-    $listref = Net::DNS::ZoneFile->parse( \$string, $include_dir );
+ $listref = Net::DNS::ZoneFile->parse( $string );
+ $listref = Net::DNS::ZoneFile->parse( $string, $include_dir );
+ $listref = Net::DNS::ZoneFile->parse( \$string );
+ $listref = Net::DNS::ZoneFile->parse( \$string, $include_dir );
 
 parse() interprets the text in the argument string
 and returns a reference to the list of Net::DNS::RR objects.
@@ -346,9 +348,10 @@ The return value is undefined if an error is encountered by the parser.
 
 sub parse {
 	my ($arg1) = @_;
-	shift if !ref($arg1) && $arg1 eq __PACKAGE__;
-	my $text = shift;
-	return &readfh( Net::DNS::ZoneFile::Text->new($text), @_ );
+	shift if $arg1 eq __PACKAGE__;
+	my $string  = shift;
+	my @include = grep {defined} shift;
+	return &readfh( Net::DNS::ZoneFile::Text->new($string), @include );
 }
 
 
@@ -365,9 +368,6 @@ sub parse {
 		my ( $class, $range, $template, $line ) = @_;
 		my $self = bless {}, $class;
 
-		$template =~ s/\\\$/\\036/g;			# disguise escaped dollar
-		$template =~ s/\$\$/\\036/g;			# disguise escaped dollar
-
 		my ( $bound, $step ) = split m#[/]#, $range;	# initial iterator state
 		my ( $first, $last ) = split m#[-]#, $bound;
 		$first ||= 0;
@@ -376,7 +376,12 @@ sub parse {
 		$step = ( $last < $first ) ? -abs($step) : abs($step);
 		$self->{count} = int( ( $last - $first ) / $step ) + 1;
 
-		@{$self}{qw(instant step template line)} = ( $first, $step, $template, $line );
+		for ($template) {
+			s/\\\$/\\036/g;				# disguise escaped dollar
+			s/\$\$/\\036/g;				# disguise escaped dollar
+			s/^"(.*)"$/$1/s;			# unwrap BIND's quoted template
+			@{$self}{qw(instant step template line)} = ( $first, $step, $_, $line );
+		}
 		return $self;
 	}
 
@@ -432,8 +437,8 @@ sub _generate {				## expand $GENERATE into input stream
 
 	my $handle = Net::DNS::ZoneFile::Generator->new( $range, $template, $self->line );
 
-	delete $self->{latest};					# forget previous owner
 	$self->{parent} = bless {%$self}, ref($self);		# save state, create link
+	delete $self->{latest};					# forget current domain name
 	return $self->{filehandle} = $handle;
 }
 
@@ -493,9 +498,7 @@ sub _getline {				## get line from current source
 		} elsif (/^\$GENERATE/) {			# directive
 			my ( $keyword, $range, @template ) = @token;
 			die '$GENERATE incomplete' unless @template;
-			$_ = join ' ', @template;
-			s/\\034/"/g if s/^"(.*)"$/$1/;		# unwrap BIND's quoted template
-			$fh = $self->_generate( $range, $_ );
+			$fh = $self->_generate( $range, "@template" );
 
 		} elsif (/^\$ORIGIN/) {				# directive
 			my ( $keyword, $origin ) = @token;
@@ -539,29 +542,28 @@ sub _getRR {				## get RR from current source
 	$self->{class} = $rr->class unless $self->{class};	# propagate RR class
 	$rr->class( $self->{class} );
 
-	$self->{TTL} ||= $rr->minimum if $rr->type eq 'SOA';	# default TTL
-	$rr->{'ttl'} = $self->{TTL} unless defined $rr->{'ttl'};
+	unless ( defined $self->{TTL} ) {
+		$self->{TTL} = $rr->minimum if $rr->type eq 'SOA';    # default TTL
+	}
+	$rr->{ttl} = $self->{TTL} unless defined $rr->{ttl};
 
 	return $self->{latest} = $rr;
 }
 
 
 sub _include {				## open $INCLUDE file
-	my $self = shift;
-	my $file = _filename(shift);
-	my $root = shift;
+	my ( $self, $include, $origin ) = @_;
 
-	my $opened = {%{$self->{fileopen}}};
-	die qq(\$INCLUDE $file: Unexpected recursion) if $opened->{$file}++;
+	my $filename = _filename($include);
+	die qq(\$INCLUDE $filename: Unexpected recursion) if $self->{fileopen}->{$filename}++;
 
-	my $discipline = PERLIO ? join( ':', '<', PerlIO::get_layers $self->{filehandle} ) : '<';
-	my $filehandle = IO::File->new( $file, $discipline ) or die qq(\$INCLUDE $file: $!);
+	my $discipline = join( ':', '<', PerlIO::get_layers $self->{filehandle} );
+	my $filehandle = IO::File->new( $filename, $discipline ) or die qq(\$INCLUDE $filename: $!);
 
-	delete $self->{latest};					# forget previous owner
 	$self->{parent} = bless {%$self}, ref($self);		# save state, create link
-	$self->_origin($root) if $root;
-	$self->{filename} = $file;
-	$self->{fileopen} = $opened;
+	delete $self->{latest};					# forget current domain name
+	$self->_origin($origin) if $origin;
+	$self->{filename} = $filename;
 	return $self->{filehandle} = $filehandle;
 }
 
@@ -605,7 +607,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -622,10 +624,11 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::RR>,
-RFC1035 Section 5.1, RFC2308
+L<perl> L<Net::DNS> L<Net::DNS::RR>
+L<RFC1035(5.1)|https://iana.org/go/rfc1035/section-5.1>
+L<RFC2308(4)|https://iana.org/go/rfc2308/section-4>
 
-L<BIND Administrator Reference Manual|http://bind.isc.org/>
+L<BIND Administrator Reference Manual|https://bind9.readthedocs.io/en/latest/>
 
 =cut
 
